import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:rook_sdk_core/rook_sdk_core.dart';
import 'package:rook_sdk_health_connect/rook_sdk_health_connect.dart';
import 'package:rook_sdk_health_connect/src/data/proto/protos.pb.dart';
import 'package:rook_sdk_health_connect/src/platform/rook_sdk_health_connect_method_channel.dart';

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();

  final platform = MethodChannelRookSdkHealthConnect();
  const channel = MethodChannel('rook_sdk_health_connect');

  tearDown(() {
    TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger.setMockMethodCallHandler(channel, null);
  });

  group('MethodChannelRookSdkHealthConnect | Result.success unwrap', () {
    setUp(() {
      TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger
          .setMockMethodCallHandler(channel, _successHandler);
    });

    test('GIVEN a Result.success WHEN setLocalLoggingLevel THEN complete', () async {
      final future = platform.setLocalLoggingLevel(LocalLoggingLevel.advanced);

      await expectLater(future, completes);
    });

    test('GIVEN a Result.success WHEN setConfiguration THEN complete', () async {
      final rookConfiguration = RookConfiguration(
        'rookUrl',
        'clientUUID',
        'clientPassword',
      );
      final future = platform.setConfiguration(rookConfiguration);

      await expectLater(future, completes);
    });

    test('GIVEN a Result.success WHEN initRook THEN complete', () async {
      final future = platform.initRook();

      await expectLater(future, completes);
    });

    test('GIVEN a Result.success WHEN updateUserID THEN complete', () async {
      final future = platform.updateUserID('userID');

      await expectLater(future, completes);
    });

    test('GIVEN a Result.success WHEN clearUserID THEN complete', () async {
      final future = platform.clearUserID();

      await expectLater(future, completes);
    });

    test(
      'GIVEN a Result.success WHEN syncUserTimeZone THEN complete',
          () async {
        final future = platform.syncUserTimeZone();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN openHealthConnectSettings THEN complete',
      () async {
        final future = platform.openHealthConnectSettings();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN hasAllPermissions THEN complete with expected value',
      () async {
        final future = platform.hasAllPermissions();

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN a Result.success WHEN requestAllPermissions THEN complete',
      () async {
        final future = platform.requestAllPermissions();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN hasSleepPermissions THEN complete with expected value',
      () async {
        final future = platform.hasSleepPermissions();

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN a Result.success WHEN requestSleepPermissions THEN complete',
      () async {
        final future = platform.requestSleepPermissions();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN hasPhysicalPermissions THEN complete with expected value',
      () async {
        final future = platform.hasPhysicalPermissions();

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN a Result.success WHEN requestPhysicalPermissions THEN complete',
      () async {
        final future = platform.requestPhysicalPermissions();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN hasBodyPermissions THEN complete with expected value',
      () async {
        final future = platform.hasBodyPermissions();

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN a Result.success WHEN requestBodyPermissions THEN complete',
      () async {
        final future = platform.requestBodyPermissions();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN shouldSyncSleepSummariesFor THEN complete with expected value',
      () async {
        final future = platform.shouldSyncSleepSummariesFor(DateTime.now());

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN a Result.success WHEN syncSleepSummary THEN complete',
      () async {
        final future = platform.syncSleepSummary(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN shouldSyncPhysicalSummariesFor THEN complete with expected value',
      () async {
        final future = platform.shouldSyncPhysicalSummariesFor(DateTime.now());

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN a Result.success WHEN syncSleepSummary THEN complete',
      () async {
        final future = platform.syncPhysicalSummary(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN shouldSyncBodySummariesFor THEN complete with expected value',
      () async {
        final future = platform.shouldSyncBodySummariesFor(DateTime.now());

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN a Result.success WHEN syncBodySummary THEN complete',
      () async {
        final future = platform.syncBodySummary(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncPendingSummaries THEN complete',
      () async {
        final future = platform.syncPendingSummaries();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncPhysicalEvents THEN complete',
      () async {
        final future = platform.syncPhysicalEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncBloodGlucoseEvents THEN complete',
      () async {
        final future = platform.syncBloodGlucoseEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncBloodPressureEvents THEN complete',
      () async {
        final future = platform.syncBloodPressureEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncBodyMetricsEvents THEN complete',
      () async {
        final future = platform.syncBodyMetricsEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncBodyHeartRateEvents THEN complete',
      () async {
        final future = platform.syncBodyHeartRateEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncPhysicalHeartRateEvents THEN complete',
      () async {
        final future = platform.syncPhysicalHeartRateEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncHydrationEvents THEN complete',
      () async {
        final future = platform.syncHydrationEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncNutritionEvents THEN complete',
      () async {
        final future = platform.syncNutritionEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncBodyOxygenationEvents THEN complete',
      () async {
        final future = platform.syncBodyOxygenationEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncPhysicalOxygenationEvents THEN complete',
      () async {
        final future = platform.syncPhysicalOxygenationEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncTemperatureEvents THEN complete',
      () async {
        final future = platform.syncTemperatureEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN a Result.success WHEN syncPendingEvents THEN complete',
      () async {
        final future = platform.syncPendingEvents();

        await expectLater(future, completes);
      },
    );
  });

  group('MethodChannelRookSdkHealthConnect | Result.exception unwrap', () {
    setUp(() {
      TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger
          .setMockMethodCallHandler(channel, _exceptionHandler);
    });

    test('GIVEN a Result.exception WHEN initRook THEN throw exception', () async {
      final future = platform.initRook();

      await expectLater(future, throwsA(isException));
    });

    test('GIVEN a Result.exception WHEN updateUserID THEN throw exception', () async {
      final future = platform.updateUserID('userID');

      await expectLater(future, throwsA(isException));
    });

    test(
      'GIVEN a Result.exception WHEN syncUserTimeZone throw exception',
          () async {
        final future = platform.syncUserTimeZone();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN openHealthConnectSettings THEN throw exception',
      () async {
        final future = platform.openHealthConnectSettings();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN hasAllPermissions THEN throw exception',
      () async {
        final future = platform.hasAllPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN requestAllPermissions THEN throw exception',
      () async {
        final future = platform.requestAllPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN hasSleepPermissions THEN throw exception',
      () async {
        final future = platform.hasSleepPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN requestSleepPermissions THEN throw exception',
      () async {
        final future = platform.requestSleepPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN hasBodyPermissions THEN throw exception',
      () async {
        final future = platform.hasBodyPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN requestBodyPermissions THEN throw exception',
      () async {
        final future = platform.requestBodyPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN hasPhysicalPermissions THEN throw exception',
      () async {
        final future = platform.hasPhysicalPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN requestPhysicalPermissions THEN throw exception',
      () async {
        final future = platform.requestPhysicalPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN shouldSyncSleepSummariesFor THEN throw exception',
      () async {
        final future = platform.shouldSyncSleepSummariesFor(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncSleepSummary throw exception',
      () async {
        final future = platform.syncSleepSummary(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN shouldSyncPhysicalSummariesFor THEN throw exception',
      () async {
        final future = platform.shouldSyncPhysicalSummariesFor(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncSleepSummary throw exception',
      () async {
        final future = platform.syncPhysicalSummary(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN shouldSyncBodySummariesFor THEN throw exception',
      () async {
        final future = platform.shouldSyncBodySummariesFor(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncBodySummary THEN throw exception',
      () async {
        final future = platform.syncBodySummary(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncPendingSummaries THEN throw exception',
      () async {
        final future = platform.syncPendingSummaries();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncPhysicalEvents THEN throw exception',
      () async {
        final future = platform.syncPhysicalEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncBloodGlucoseEvents THEN throw exception',
      () async {
        final future = platform.syncBloodGlucoseEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncBloodPressureEvents THEN throw exception',
      () async {
        final future = platform.syncBloodPressureEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncBodyMetricsEvents THEN throw exception',
      () async {
        final future = platform.syncBodyMetricsEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncBodyHeartRateEvents THEN throw exception',
      () async {
        final future = platform.syncBodyHeartRateEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncPhysicalHeartRateEvents THEN throw exception',
      () async {
        final future = platform.syncPhysicalHeartRateEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncHydrationEvents THEN throw exception',
      () async {
        final future = platform.syncHydrationEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncNutritionEvents THEN throw exception',
      () async {
        final future = platform.syncNutritionEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncBodyOxygenationEvents THEN throw exception',
      () async {
        final future = platform.syncBodyOxygenationEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncPhysicalOxygenationEvents THEN throw exception',
      () async {
        final future = platform.syncPhysicalOxygenationEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncTemperatureEvents THEN throw exception',
      () async {
        final future = platform.syncTemperatureEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN a Result.exception WHEN syncPendingEvents THEN throw exception',
      () async {
        final future = platform.syncPendingEvents();

        await expectLater(future, throwsA(isException));
      },
    );
  });

  group('MethodChannelRookSdkHealthConnect checkAvailability', () {
    group('Valid AvailabilityStatusProto value', () {
      setUp(() {
        TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger.setMockMethodCallHandler(
          channel,
          (message) async {
            return AvailabilityStatusProto.INSTALLED.value;
          },
        );
      });

      test('GIVEN the happy path WHEN checkAvailability THEN complete with expected value', () async {
        final future = platform.checkAvailability();

        await expectLater(future, completion(HCAvailabilityStatus.installed));
      });
    });

    group('Invalid AvailabilityStatusProto value', () {
      setUp(() {
        TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger.setMockMethodCallHandler(
          channel,
          (message) async {
            return 1000;
          },
        );
      });

      test('GIVEN the unhappy path WHEN checkAvailability THEN complete with HCAvailabilityStatus.notSupported',
          () async {
        final future = platform.checkAvailability();

        await expectLater(future, completion(HCAvailabilityStatus.notSupported));
      });
    });
  });
}

Future<Object?>? _successHandler(MethodCall methodCall) async {
  final proto = ResultBooleanProto();
  proto.success = true;
  return proto.writeToBuffer();
}

Future<Object?>? _exceptionHandler(MethodCall methodCall) async {
  final proto = ResultBooleanProto();
  proto.genericExceptionProto = GenericExceptionProto(message: 'Generic error');

  return proto.writeToBuffer();
}
